from . import user
from . import exception, utils
import requests
import re
import json
from .utils import Get, Post
import math

headers = utils.default_headers
apis = utils.get_apis()


class SendDynamic:
    def __init__(self, text: str, verify: utils.Verify):
        self.verify = verify
        self._text = ""
        self._ctrl = None
        self._origin_text = text
        self.set_text(text)

    def set_text(self, text: str):
        pattern = re.compile(r"@\d*@")
        match = re.findall(pattern, text)
        at_users = {}
        for m in match:
            uid = m.replace("@", "")
            info = user.UserInfo(uid).get_info()
            at_users[uid] = info["name"]
        new_text = text
        for u in at_users.items():
            user_pattern = re.compile("@%s@" % u[0])
            new_text = re.sub(user_pattern, "@%s" % u[1], new_text)
        ctrl = []
        for u in at_users.items():
            user_pattern = "@%s" % u[1]
            length = 2 + len(u[1])
            location = new_text.find(user_pattern)
            at = {
                "location": location,
                "type": 1,
                "length": length,
                "data": str(u[0])
            }
            ctrl.append(at)
        self._ctrl = ctrl
        self._text = new_text
        self._origin_text = text

    def get_text(self):
        return self._text

    def get_origin_text(self):
        return self._origin_text

    def get_at_uids(self):
        at_uids = [i["data"] for i in self._ctrl]
        at_uids_str = ""
        if len(at_uids) > 0:
            i = 0
            for uid in at_uids:
                at_uids_str += str(uid)
                i += 1
                if i < len(at_uids):
                    at_uids_str += ","
        return at_uids_str


class UploadImages:
    def __init__(self, images_path: list, verify: utils.Verify):
        if type(verify) != utils.Verify:
            raise Exception("请传入Verify类")
        else:
            self.verify = verify
        if len(images_path) > 9:
            raise Exception("图片最多 9 张")
        elif type(images_path) != list:
            raise Exception("图片必须传入list")
        else:
            self.__images_path = images_path
        self.bilibili_path = []

    def add_image(self, image_path: str):
        if len(self.__images_path) >= 9:
            raise Exception("图片最多 9 张")
        else:
            self.__images_path.append(image_path)

    def del_image(self, image_path: str):
        if len(self.__images_path) == 0:
            raise Exception("没有图片")
        else:
            self.__images_path.remove(image_path)

    def pop_image(self, index: int):
        self.__images_path.pop(index)

    def get_images(self):
        return self.__images_path

    def clear_images(self):
        self.__images_path = []

    def upload(self):
        api = apis["dynamic"]["send"]["upload_img"]
        data = {
            "biz": "draw",
            "category": "daily"
        }
        for img in self.__images_path:
            files = {
                "file_up": open(img, "rb")
            }
            print("Uploading", img)
            req = requests.post(url=api["url"], data=data, files=files, cookies=self.verify.get_cookies())
            if req.ok:
                con = req.json()
                if con["code"] != 0:
                    raise exception.BiliException(con["code"], con["message"])
                else:
                    pic = {
                        "img_src": con["data"]["image_url"],
                        "img_height": con["data"]["image_height"],
                        "img_width": con["data"]["image_width"]
                    }
                    self.bilibili_path.append(pic)
            else:
                raise exception.NetworkException(req.status_code)


class TextDynamic(SendDynamic):
    def __init__(self, text: str, verify: utils.Verify):
        super().__init__(text, verify)

    def get_data(self):
        data = {
            "dynamic_id": 0,
            "type": 4,
            "rid": 0,
            "content": self._text,
            "extension": "{\"emoji_type\":1}",
            "at_uids": self.get_at_uids(),
            "ctrl": json.dumps(self._ctrl)
        }
        return data


class DrawDynamic(SendDynamic):
    def __init__(self, text: str, upload_images: UploadImages, verify: utils.Verify):
        super().__init__(text, verify)
        self.upload_images = upload_images

    def get_data(self):
        data = {
            "biz": 3,
            "category": 3,
            "type": 0,
            "title": "",
            "tags": "",
            "pictures": json.dumps(self.upload_images.bilibili_path),
            "description": self._text,
            "content": self._text,
            "from": "create.dynamic.web",
            "extension": "{\"emoji_type\":1}",
            "at_uids": self.get_at_uids(),
            "at_control": json.dumps(self._ctrl)
        }
        return data

    def upload(self):
        self.upload_images.upload()


# 如果要@人，格式为 “@用户UID@ ”
class ScheduleDynamic:
    def __init__(self, dynamic: (TextDynamic, DrawDynamic), timestamp: int):
        self.timestamp = timestamp
        self.dynamic = dynamic

    def send(self):
        if type(self.dynamic) == DrawDynamic:
            type_ = 4
            self.dynamic.upload()
        elif type(self.dynamic) == TextDynamic:
            type_ = 2
        else:
            raise Exception("未知的动态类型")
        data = {"type": type_, "publish_time": self.timestamp, "request": json.dumps(self.dynamic.get_data()),
                "csrf_token": self.dynamic.verify.csrf}
        api = apis["dynamic"]["send"]["schedule"]
        post = Post(url=api["url"], data=data, cookies=self.dynamic.verify.get_cookies())
        post()


class InstantDynamic:
    def __init__(self, dynamic):
        if isinstance(dynamic, (DrawDynamic, TextDynamic)):
            self.dynamic = dynamic
        else:
            raise Exception("传入参数非法")

    def send(self):
        if type(self.dynamic) == DrawDynamic:
            api = apis["dynamic"]["send"]["instant_draw"]
            self.dynamic.upload()
        else:
            api = apis["dynamic"]["send"]["instant_text"]
        request = self.dynamic.get_data()
        setting = {
            "copy_forbidden": 0,
            "cachedTime": 0
        }
        data = {
            "csrf_token": self.dynamic.verify.csrf,
            "setting": json.dumps(setting)
        }
        data.update(request)
        post = Post(url=api["url"], data=data, cookies=self.dynamic.verify.get_cookies())
        post()


class DynamicInfo:
    def __init__(self, dyid: int, verify: utils.Verify = utils.Verify()):
        self.dyid = dyid
        self.info = None
        self.verify = verify

    def __get_self_info(self):
        if self.info is None:
            self.info = self.get_info()
            if self.info["desc"]["type"] == 2:
                self.oid = self.info["desc"]["rid"]
                self.is_draw = True
            else:
                self.is_draw = False

    def get_info(self):
        api = apis["dynamic"]["info"]["detail"]
        params = {
            "dynamic_id": self.dyid
        }
        get = Get(url=api["url"], params=params)
        data = get()
        data["card"]["card"] = json.loads(data["card"]["card"])
        data["card"]["extend_json"] = json.loads(data["card"]["extend_json"])
        self.info = data["card"]
        return data["card"]

    def get_reposts(self, limit: int = 560):
        def get(offset):
            api = apis["dynamic"]["info"]["repost"]
            params = {"dynamic_id": self.dyid, "offset": offset}
            g = Get(url=api["url"], params=params)
            data = g()
            if data["total"] == 0:
                return {"has_more": False, "items": []}
            items = data["items"]
            for i in items:
                i["card"] = json.loads(i["card"])
                i["extend_json"] = json.loads(i["extend_json"])
            total = data["total"]
            ret = {
                "items": items,
                "total": total
            }
            if data["has_more"] == 1:
                ret["next_offset"] = data["offset"]
                ret["has_more"] = True
            else:
                ret["has_more"] = False
            return ret

        offset = ""
        reposts_list = []
        sum_ = 0
        while True:
            reposts = get(offset)
            if reposts["has_more"]:
                offset = reposts["next_offset"]
            reposts_list += reposts["items"]
            sum_ += len(reposts["items"])
            if not reposts["has_more"] or sum_ >= 560 or sum_ >= limit:
                break
        return reposts_list[:limit]

    def get_replies(self, limit: int = 114514):
        def get(pn):
            api = apis["dynamic"]["info"]["reply"]
            if self.is_draw:
                ty = 11
                oid = self.oid
            else:
                ty = 17
                oid = self.dyid
            params = {
                "pn": pn,
                "type": ty,
                "oid": oid,
                "sort": 0
            }
            g = Get(url=api["url"], params=params)
            data = g()
            return data
        if self.info is None:
            self.__get_self_info()
        replies = []
        first = get(0)
        total_pages = math.ceil(first["page"]["count"] / first["page"]["size"])
        replies += first["replies"]
        sum_ = len(first["replies"])
        if sum_ < limit and total_pages > 1:
            for page in range(2, total_pages):
                data = get(page)
                replies += data["replies"]
                sum_ += len(data["replies"])
                if sum_ >= limit:
                    break
        return replies[:limit]


class DynamicOperate:
    def __init__(self, dyid: int, verify: utils.Verify):
        self.dyid = dyid
        self.verify = verify
        self.__uid = None
        self.__info = None

    def __get_self_info(self):
        if self.__info is None:
            self.__info = DynamicInfo(dyid=self.dyid).get_info()
            if self.__info["desc"]["type"] == 2:
                self.oid = self.__info["desc"]["rid"]
                self.__is_draw = True
            else:
                self.__is_draw = False

    def like(self, mode: bool = True):
        api = apis["dynamic"]["operate"]["like"]
        if self.__uid is None:
            my_info = Get(url=apis["user"]["info"]["my_info"]["url"], cookies=self.verify.get_cookies())
            self.__uid = my_info()["mid"]
        if mode:
            up = 1
        else:
            up = 2
        data = {
            "dynamic_id": self.dyid,
            "up": up,
            "uid": self.__uid
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()

    def reply(self, text: str):
        api = apis["dynamic"]["operate"]["reply"]
        self.__get_self_info()
        if self.__is_draw:
            type_ = 11
            oid = self.oid
        else:
            type_ = 17
            oid = self.dyid

        data = {
            "oid": oid,
            "type": type_,
            "message": text,
            "plat": 1,
            "csrf": self.verify.csrf
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()

    def delete(self):
        api = apis["dynamic"]["operate"]["delete"]
        data =  {
            "dynamic_id": self.dyid
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()

    def repost(self, text: str):
        api = apis["dynamic"]["operate"]["repost"]
        data = {
            "dynamic_id": self.dyid,
            "content": text,
            "extension": "{\"emoji_type\":1}",
            "csrf_token": self.verify.csrf
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()


import json
from . import utils
import math
from .utils import Get, Post

headers = utils.default_headers
apis = utils.get_apis()


# 把你资料给我交了（迫真）
class UserInfo:
    def __init__(self, uid: int, verify: utils.Verify = utils.Verify()):
        self.uid = uid
        self.verify = verify

    def get_info(self):
        api = apis["user"]["info"]["info"]
        api1 = apis["user"]["info"]["stat"]
        params = {
            "mid": self.uid
        }
        params1 = {
            "vmid": self.uid
        }
        get = Get(url=api["url"], params=params)
        get1 = Get(url=api1["url"], params=params1)
        data = get()
        data.update(get1())
        return data

    def get_live_info(self):
        api = apis["user"]["info"]["live"]
        params = {
            "mid": self.uid
        }
        get = Get(url=api["url"], params=params)
        con = get()
        return con

    # sort: 0-上传日期，1-播放量，2-收藏量
    def get_video(self, sort: int = 0, limit: int = 114514):
        def get(page):
            if sort == 0:
                sort_str = "pubdate"
            elif sort == 1:
                sort_str = "click"
            else:
                sort_str = "stow"
            api = apis["user"]["info"]["video"]
            params = {
                "mid": self.uid,
                "ps": 30,
                "tid": 0,
                "pn": page,
                "keyword": "",
                "order": sort_str
            }
            get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        first_page = get(1)
        count = 0
        if first_page["page"]["count"] > first_page["page"]["ps"]:
            pages = math.ceil(first_page["page"]["count"] / first_page["page"]["ps"])
            video_list = first_page["list"]["vlist"]
            count += len(video_list)
            if count < limit:
                for page in range(2, pages + 1):
                    data = get(page)["list"]["vlist"]
                    video_list += data
                    count += len(data)
                    if count > limit:
                        break
            return video_list[:limit]
        else:
            return first_page["list"][:limit]

    # sort: 0-上传日期，1-播放量，2-收藏量
    def get_audio(self, sort: int = 0, limit: int = 114514):
        def get(page):
            if sort == 0:
                sort_str = 1
            elif sort == 1:
                sort_str = 2
            else:
                sort_str = 3
            api = apis["user"]["info"]["audio"]
            params = {
                "uid": self.uid,
                "ps": 30,
                "pn": page,
                "order": sort_str
            }
            get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        count = 0
        first_page = get(1)
        page_count = first_page["pageCount"]
        count += page_count
        if first_page["pageCount"] > 1 and count < limit:
            audio_list = first_page["data"]
            for page in range(2, page_count + 1):
                data = get(page)["data"]
                count += len(data)
                audio_list += data
                if count >= limit:
                    break
            return audio_list[:limit]
        else:
            return first_page["data"][:limit]

    def get_article(self, sort: int = 0, limit: int = 114514):
        def get(page):
            if sort == 0:
                sort_str = "publish_time"
            elif sort == 1:
                sort_str = "view"
            else:
                sort_str = "fav"
            api = apis["user"]["info"]["article"]
            params = {
                "mid": self.uid,
                "ps": 30,
                "pn": page,
                "sort": sort_str
            }
            get1 = Get(url=api["url"], params=params)
            con = get1()
            return con

        first_page = get(1)
        page_count = math.ceil(first_page["count"] / first_page["ps"])
        count = len(first_page["articles"])
        if page_count > 1 and count < limit:
            article_list = first_page["articles"]
            for page in range(2, page_count + 1):
                data = get(page)["articles"]
                article_list += data
                count += len(data)
                if count >= limit:
                    break
            return article_list[:limit]
        else:
            return first_page["articles"][:limit]

    # sort: 0最近更新，1最多阅读
    def get_article_list(self, sort: int = 0):
        api = apis["user"]["info"]["article_lists"]
        params = {
            "mid": self.uid,
            "sort": sort
        }
        get = Get(url=api["url"], params=params)
        con = get()
        return con

    def get_dynamic(self, limit: int = 114514):
        def get(offset):
            api = apis["user"]["info"]["dynamic"]
            params = {
                "host_uid": self.uid,
                "offset_dynamic_id": offset,
                "need_top": 0
            }
            get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        p = 0
        first = get(0)
        cards = first["cards"]
        next_offset = first["next_offset"]
        has_more = first["has_more"]
        count = limit
        while has_more == 1 and count > 0:
            p += 1
            this = get(next_offset)
            next_offset = this["next_offset"]
            cards += this["cards"]
            has_more = this["has_more"]
            count -= len(this["cards"])
        return cards[:limit]

    # "type": "1追番，2追剧"
    def get_bangumi(self, type_: int = 1):
        def get(page):
            api = apis["user"]["info"]["bangumi"]
            params = {
                "vmid": self.uid,
                "pn": page,
                "ps": 15,
                "type": type_
            }
            if self.verify.has_sess():
                get1 = Get(url=api["url"], params=params, cookies=self.verify.get_cookies())
            else:
                get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        first = get(1)
        if first["total"] > first["ps"]:
            pages = math.ceil(first["total"] / first["ps"])
            bangumi = first["list"]
            for p in range(2, pages + 1):
                g = get(p)
                bangumi += g["list"]
            return bangumi
        else:
            return first["list"]

    def get_media_list(self):
        def get(page):
            api = apis["user"]["info"]["media_list"]
            params = {
                "up_mid": self.uid,
                "pn": page,
                "ps": 100,
                "is_space": 0
            }
            if self.verify.has_sess():
                get1 = Get(url=api["url"], params=params, cookies=self.verify.get_cookies())
            else:
                get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        first = get(1)
        if first["count"] > 100:
            pages = math.ceil(first["count"] / 100.0)
            me = first["list"]
            for p in range(2, pages + 1):
                g = get(p)
                me += g["list"]
            return me
        else:
            return first["list"]

    def get_media_list_content(self, media_id: int, sort: int = 0, limit=114514):
        def get(page):
            if sort == 0:
                sort_str = "mtime"
            elif sort == 1:
                sort_str = "view"
            else:
                sort_str = "pubtime"
            api = apis["user"]["info"]["media_list_content"]
            params = {
                "media_id": media_id,
                "pn": page,
                "ps": 20,
                "order": sort_str,
                "keyword": "",
                "type": 0,
                "tid": 0
            }
            if self.verify.has_sess():
                get1 = Get(url=api["url"], params=params, cookies=self.verify.get_cookies())
            else:
                get1 = Get(url=api["url"], params=params)
            con = get1()
            return con
        first = get(1)
        ret = first
        count = 0
        if first["info"]["media_count"] > 20:
            pages = math.ceil(first["info"]["media_count"] / 20.0)
            me = first["medias"]
            count += len(first["medias"])
            if count < limit:
                for p in range(2, pages + 1):
                    g = get(p)
                    me += g["medias"]
                    count += len(g["medias"])
                    if count >= limit:
                        break
                ret["medias"] = me[:limit]
            return ret
        else:
            return ret


class UserOperate:
    def __init__(self, uid: int, verify: utils.Verify = utils.Verify()):
        self.uid = uid
        self.verify = verify
        self.__my_info = Get(url=apis["user"]["info"]["my_info"]["url"], cookies=self.verify.get_cookies())()

    def subscribe(self, mode: bool = True):
        api = apis["user"]["operate"]["modify"]
        if mode == 0:
            act = 2
        else:
            act = 1
        data = {
            "fid": self.uid,
            "act": act,
            "re_src": 11,
            "csrf": self.verify.csrf
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()

    def send_msg(self, text: str):
        api = apis["user"]["operate"]["send_msg"]
        data = {
            "msg[sender_uid]": self.__my_info["mid"],
            "msg[receiver_id]": self.uid,
            "msg[receiver_type]": 1,
            "msg[msg_type]": 1,
            "msg[msg_status]": 0,
            "msg[content]": ""
        }
        msg = {
            "content": text
        }
        data["msg[content]"] = json.dumps(msg)
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()

    def set_black(self, mode: bool = True):
        api = apis["user"]["operate"]["modify"]
        if mode == 0:
            act = 6
        else:
            act = 5
        data = {
            "fid": self.uid,
            "act": act,
            "re_src": 11,
            "csrf": self.verify.csrf
        }
        post = Post(url=api["url"], data=data, cookies=self.verify.get_cookies())
        post()


